﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GpsCardGatewayPosition.Common.Helper
{
    public class GeoConvert
    {
        const double Pi = 3.14159265358979324;
        const double A = 6378245.0;
        const double Ee = 0.00669342162296594323;
        const double XPi = 3.14159265358979324 * 3000.0 / 180.0;
        //
        /// <summary>
        ///  地球坐标系 ===> 火星坐标系统 (gps === > google)
        /// </summary>
        /// <param name="wgLat"></param>
        /// <param name="wgLon"></param>
        /// <param name="mgLat"></param>
        /// <param name="mgLon"></param>
        public static void Transform(double wgLat, double wgLon, out double mgLat, out double mgLon)
        {
            if (OutOfChina(wgLat, wgLon))
            {
                mgLat = wgLat;
                mgLon = wgLon;
                return;
            }
            var dLat = TransformLat(wgLon - 105.0, wgLat - 35.0);
            var dLon = TransformLon(wgLon - 105.0, wgLat - 35.0);
            var radLat = wgLat / 180.0 * Pi;
            var magic = Math.Sin(radLat);
            magic = 1 - Ee * magic * magic;
            var sqrtMagic = Math.Sqrt(magic);
            dLat = (dLat * 180.0) / ((A * (1 - Ee)) / (magic * sqrtMagic) * Pi);
            dLon = (dLon * 180.0) / (A / sqrtMagic * Math.Cos(radLat) * Pi);
            mgLat = wgLat + dLat;
            mgLon = wgLon + dLon;
        }

        /// <summary>
        /// 算法纠偏，无缓存
        /// 0:高德纬度
        /// 1:高德经度
        /// 2:百度纬度
        /// 3:百度经度
        /// </summary>
        /// <param name="lat"></param>
        /// <param name="lng"></param>
        /// <returns></returns>
        public static decimal[] ConvertGoogleBaiduLatLng(decimal lat, decimal lng)
        {
            //中国经度范围：73°33′E至135°05′E 纬度范围：3°51′N至53°33′N
            var LatLng = new decimal[] { lat, lng, lat, lng };
            if (lat > 4 && lat < 54 && lng > 73 && lng < 136)
            {
                try
                {
                    double gLat = 0, gLng = 0, bLat = 0, bLng = 0;
                    Transform(SafeType.SafeDouble(lat), SafeType.SafeDouble(lng), out gLat, out gLng);
                    G2B(gLat, gLng, out bLat, out bLng);
                    LatLng[0] = Convert.ToDecimal(gLat);
                    LatLng[1] = Convert.ToDecimal(gLng);
                    LatLng[2] = Convert.ToDecimal(bLat);
                    LatLng[3] = Convert.ToDecimal(bLng);
                }
                catch (Exception)
                {
                    //LogHelper.WriteLog($"ConvertGoogleBaiduLatLng[异常] {ex.Message}");
                }
            }
            return LatLng;
        }


        /// <summary>
        /// google to gps
        /// </summary>
        /// <param name="lat"></param>
        /// <param name="lng"></param>
        /// <param name="olat"></param>
        /// <param name="olng"></param>
        public static void G2Gps(double lat, double lng, out double olat, out double olng)
        {
            if (OutOfChina(lat, lng))
            {
                olat = lat;
                olng = lng;
                return;
            }

            var dLat = TransformLat(lng - 105.0, lat - 35.0);
            var dLng = TransformLon(lng - 105.0, lat - 35.0);
            var radLat = lat / 180.0 * Math.PI;
            var magic = Math.Sin(radLat);
            magic = 1 - Ee * magic * magic;
            var sqrtMagic = Math.Sqrt(magic);
            dLat = (dLat * 180.0) / ((A * (1 - Ee)) / (magic * sqrtMagic) * Math.PI);
            dLng = (dLng * 180.0) / (A / sqrtMagic * Math.Cos(radLat) * Math.PI);
            olat = lat - dLat;
            olng = lng - dLng;
        }

        /// <summary>
        /// 判断是否在中国
        /// </summary>
        /// <param name="lat"></param>
        /// <param name="lon"></param>
        /// <returns></returns>
        static bool OutOfChina(double lat, double lon)
        {
            return !ChinaPolygon.IsInChina(lat, lon);
        }

        static double TransformLat(double x, double y)
        {
            var ret = -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * y + 0.1 * x * y + 0.2 * Math.Sqrt(Math.Abs(x));
            ret += (20.0 * Math.Sin(6.0 * x * Pi) + 20.0 * Math.Sin(2.0 * x * Pi)) * 2.0 / 3.0;
            ret += (20.0 * Math.Sin(y * Pi) + 40.0 * Math.Sin(y / 3.0 * Pi)) * 2.0 / 3.0;
            ret += (160.0 * Math.Sin(y / 12.0 * Pi) + 320 * Math.Sin(y * Pi / 30.0)) * 2.0 / 3.0;
            return ret;
        }

        static double TransformLon(double x, double y)
        {
            var ret = 300.0 + x + 2.0 * y + 0.1 * x * x + 0.1 * x * y + 0.1 * Math.Sqrt(Math.Abs(x));
            ret += (20.0 * Math.Sin(6.0 * x * Pi) + 20.0 * Math.Sin(2.0 * x * Pi)) * 2.0 / 3.0;
            ret += (20.0 * Math.Sin(x * Pi) + 40.0 * Math.Sin(x / 3.0 * Pi)) * 2.0 / 3.0;
            ret += (150.0 * Math.Sin(x / 12.0 * Pi) + 300.0 * Math.Sin(x / 30.0 * Pi)) * 2.0 / 3.0;
            return ret;
        }

        /// <summary>
        /// Google to Baidu
        /// </summary>
        /// <param name="glat"></param>
        /// <param name="glng"></param>
        /// <param name="blat"></param>
        /// <param name="blng"></param>
        public static void G2B(double glat, double glng, out double blat, out double blng)
        {
            var x = glng;
            var y = glat;
            var z = Math.Sqrt(x * x + y * y) + 0.00002 * Math.Sin(y * XPi);
            var theta = Math.Atan2(y, x) + 0.000003 * Math.Cos(x * XPi);
            blng = z * Math.Cos(theta) + 0.0065;
            blat = z * Math.Sin(theta) + 0.006;
        }

        ///// <summary>
        ///// Baidu to Google
        ///// </summary>
        ///// <param name="blat"></param>
        ///// <param name="blng"></param>
        ///// <param name="glat"></param>
        ///// <param name="glng"></param>
        //public static void B2G(double blat, double blng, out double glat, out double glng)
        //{
        //    var x = blng - 0.0065;
        //    var y = blat - 0.006;
        //    var z = Math.Sqrt(x * x + y * y) - 0.00002 * Math.Sin(y * XPi);
        //    var theta = Math.Atan2(y, x) - 0.000003 * Math.Cos(x * XPi);
        //    glng = z * Math.Cos(theta);
        //    glat = z * Math.Sin(theta);
        //}
        ///// <summary>
        ///// 百度转原始经纬度
        ///// </summary>
        ///// <param name="blat"></param>
        ///// <param name="blng"></param>
        ///// <param name="olat"></param>
        ///// <param name="olng"></param>
        //public static void B2Gps(double blat, double blng, out double olat, out double olng)
        //{
        //    if (OutOfChina(blat, blng))
        //    {
        //        olat = blat;
        //        olng = blng;
        //        return;
        //    }
        //    double glat, glng;
        //    B2G(blat, blng, out glat, out glng);
        //    G2Gps(glat, glng, out olat, out olng);
        //}
        ///// <summary>
        ///// 原始转百度
        ///// </summary>
        ///// <param name="olat"></param>
        ///// <param name="olng"></param>
        ///// <param name="blat"></param>
        ///// <param name="blng"></param>
        //public static void Gps2Bd(double olat, double olng, out double blat, out double blng)
        //{
        //    double glat, glng;
        //    Transform(olat, olng, out glat, out glng);
        //    G2B(glat, glng, out blat, out blng);
        //}
    }
}
